.ais-SearchBox {
    margin-bottom: 1em;
}

.ais-RefinementList-count:before {
    content: "(";
}

.ais-RefinementList-count:after {
    content: ")";
}

.ais-SearchBox-input {
    border: 1px solid #888;
    border-radius: 0.5em;
    font-size: 1.25em;
}

.ais-RefinementList-label {
    white-space: nowrap;
}

.ais-RefinementList-list {
    list-style-type: none;
    padding-left: 1em;
}   

.ais-RefinementList-item {
}   

.hit-meta {
    font-size: 0.9em;
    color: #aaa;
}

.hit {
    margin-bottom: 1em;
}

.hit-text {
    margin-top: 0.25em;
    font-size: 0.9em;
    font-style: italic;
}

.ais-Highlight-highlighted {
    font-weight: bold;
    background: #ddd;
}


.header {
  display: flex;
  align-items: center;
  min-height: 50px;
  padding: 0.5rem 1rem;
  background-image: linear-gradient(284deg, #fedd4e, #fcb43a);
  color: #fff;
  margin-bottom: 1rem;
}

.header a {
  color: #fff;
  text-decoration: none;
}

.header-title {
  font-size: 1.2rem;
  font-weight: normal;
}

.header-title::after {
  content: ' ▸ ';
  padding: 0 0.5rem;
}

.header-subtitle {
  font-size: 1.2rem;
}

.container {
  max-width: 1200px;
  margin: 0 auto;
  padding: 1rem;
}

.search-panel {
  display: flex;
}

.search-panel__filters {
  flex: 1;
}

.search-panel__results {
  flex: 3;
}

.ais-Highlight-highlighted {
  color: inherit;
  font-size: inherit;
}

#searchbox {
  margin-bottom: 2rem;
}

#search-control {
    display: flex;
    justify-content: space-between;
    margin-bottom: 1em;
}

.ais-Pagination-item {
    margin-right: 0.25em;
    padding-top: 0.25em;
    padding-bottom: 0.25em;
    padding-left: 0.5em;
    padding-right: 0.5em;
    border: 1px solid #ddd;
    border-radius: 0.25em;
    font-size: 0.9em;
}

.ais-Hits-item {
    width: 100% !important;
}

.ais-Hits-list {
    list-style-type: circle;
}

.refinement-label {
    font-weight: bold;
}
